CharacteristicFlags

Struct CharacteristicFlags 

pub struct CharacteristicFlags {
Show 17 fields pub broadcast: bool, pub read: bool, pub write_without_response: bool, pub write: bool, pub notify: bool, pub indicate: bool, pub authenticated_signed_writes: bool, pub extended_properties: bool, pub reliable_write: bool, pub writable_auxiliaries: bool, pub encrypt_read: bool, pub encrypt_write: bool, pub encrypt_authenticated_read: bool, pub encrypt_authenticated_write: bool, pub secure_read: bool, pub secure_write: bool, pub authorize: bool,
}
Available on crate feature bluetoothd only.
Bluetooth GATT characteristic flags.

Fields

broadcast: bool

If set, permits broadcasts of the Characteristic Value using Server Characteristic Configuration Descriptor.

read: bool

If set allows clients to read this characteristic.

write_without_response: bool

If set allows clients to use the Write Request/Response operation.

write: bool

If set allows clients to use the Write Command ATT operation.

notify: bool

If set allows the client to use the Handle Value Notification operation.

indicate: bool

If set allows the client to use the Handle Value Indication/Confirmation operation.

authenticated_signed_writes: bool

If set allows clients to use the Signed Write Without Response procedure.

extended_properties: bool

Extended properties available.

reliable_write: bool

If set allows clients to use the Reliable Writes procedure.

writable_auxiliaries: bool

If set a client can write to the Characteristic User Description Descriptor.

encrypt_read: bool

Require encryption for reading.

encrypt_write: bool

Require encryption for writing.

encrypt_authenticated_read: bool

Require authentication for reading.

encrypt_authenticated_write: bool

Require authentication for writing.

secure_read: bool

Require security for reading.

secure_write: bool

Require security for writing.

authorize: bool

Authorize flag.