#[non_exhaustive]pub enum CharacteristicProperty {
Uuid(Uuid),
Notifying(bool),
Flags(CharacteristicFlags),
CachedValue(Vec<u8>),
Mtu(usize),
}bluetoothd only.Expand description
GATT characteristic property.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Uuid(Uuid)
128-bit characteristic UUID.
Notifying(bool)
Whether notifications or indications on this characteristic are supported and currently enabled.
Returns Some(true) if enabled and Some(false) if disabled.
When notifications are unsupported this returns None.
Flags(CharacteristicFlags)
Defines how the characteristic value can be used.
See Core spec “Table 3.5: Characteristic Properties bit field”, and “Table 3.8: Characteristic Extended Properties bit field”.
CachedValue(Vec<u8>)
The cached value of the characteristic.
This property gets updated only after a successful read request and when a notification or indication is received.
Mtu(usize)
The maximum transmission unit for the characteristic.
This is the maximum amount of data that can be sent or received in a single packet for this characteristic. Longer data may be able to be sent or received using long procedures when available.