pub struct CharacteristicFlags {Show 17 fields
pub broadcast: bool,
pub read: bool,
pub write_without_response: bool,
pub write: bool,
pub notify: bool,
pub indicate: bool,
pub authenticated_signed_writes: bool,
pub extended_properties: bool,
pub reliable_write: bool,
pub writable_auxiliaries: bool,
pub encrypt_read: bool,
pub encrypt_write: bool,
pub encrypt_authenticated_read: bool,
pub encrypt_authenticated_write: bool,
pub secure_read: bool,
pub secure_write: bool,
pub authorize: bool,
}bluetoothd only.Expand description
Bluetooth GATT characteristic flags.
Fields
broadcast: boolIf set, permits broadcasts of the Characteristic Value using Server Characteristic Configuration Descriptor.
read: boolIf set allows clients to read this characteristic.
write_without_response: boolIf set allows clients to use the Write Request/Response operation.
write: boolIf set allows clients to use the Write Command ATT operation.
notify: boolIf set allows the client to use the Handle Value Notification operation.
indicate: boolIf set allows the client to use the Handle Value Indication/Confirmation operation.
authenticated_signed_writes: boolIf set allows clients to use the Signed Write Without Response procedure.
extended_properties: boolExtended properties available.
reliable_write: boolIf set allows clients to use the Reliable Writes procedure.
writable_auxiliaries: boolIf set a client can write to the Characteristic User Description Descriptor.
encrypt_read: boolRequire encryption for reading.
encrypt_write: boolRequire encryption for writing.
encrypt_authenticated_read: boolRequire authentication for reading.
encrypt_authenticated_write: boolRequire authentication for writing.
secure_read: boolRequire security for reading.
secure_write: boolRequire security for writing.
Authorize flag.